package geniusweb.partiesserver.repository;

import geniusweb.actions.PartyId;
import geniusweb.party.Capabilities;
import geniusweb.party.Party;
import tudelft.utilities.repository.RepoElement;

public class AvailableParty implements RepoElement<String> {

	private final Class<? extends Party> partyclass;
	private final String name;
	private final transient Capabilities capabilities;
	private final transient String description;

	/**
	 * 
	 * @param name       the filename. This is not a {@link PartyId}.
	 * @param partyclass the class of the party.
	 * @throws ClassNotFoundException if unknown partyclass
	 */
	public AvailableParty(String name, Class<? extends Party> partyclass)
			throws ClassNotFoundException {
		this.name = name;
		this.partyclass = partyclass;

		Party party;
		try {
			party = partyclass.newInstance();
		} catch (Throwable e) {
			throw new ClassNotFoundException(
					"Could not instantiate " + partyclass, e);
		}
		capabilities = party.getCapabilities();
		description = party.getDescription();

	}

	@Override
	public String getID() {
		return name;
	}

	public String getName() {
		return name;
	}

	public Class<? extends Party> getPartyClass() {
		return partyclass;
	}

	/**
	 * 
	 * @return cached copy of what was received from a call to
	 *         {@link Party#getDescription()}.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * 
	 * @return cached copy of what was received from a call to
	 *         {@link AvailableParty#getCapabilities()}
	 */
	public Capabilities getCapabilities() {
		return capabilities;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result
				+ ((partyclass == null) ? 0 : partyclass.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AvailableParty other = (AvailableParty) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (partyclass == null) {
			if (other.partyclass != null)
				return false;
		} else if (!partyclass.equals(other.partyclass))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return name + " (" + partyclass + ")";
	}

}
